# cython: language_level=3
# distutils: language = c++
cimport cython
from libc.stdint cimport int64_t, uint64_t

cdef extern from "nazo_rand.hpp" namespace "Storm":
    void seed(uint64_t seed)
    int64_t uniform_int_variate_noargs()
    int64_t random_range(int64_t start, int64_t stop, int64_t step) nogil
    int64_t random_below(int64_t number) nogil
    int64_t uniform_int_variate(int64_t a, int64_t b) nogil
    double uniform_real_variate_noargs()
    double uniform_real_variate(double a, double b) nogil

cdef inline int64_t cy_random_below(int64_t number) nogil:
    return random_below(number)

cdef inline int64_t cy_uniform_int_variate(int64_t a, int64_t b) nogil:
    return uniform_int_variate(a, b)

cpdef int random_integer_noargs():
    return uniform_int_variate_noargs()

cpdef void shuffle(list array):
    cdef int i, j
    cdef object temp
    cdef int length = len(array)
    for i in range(length - 1, 0, -1):
        j = cy_uniform_int_variate(0, i)
        temp = array[i]
        array[i] = array[j]
        array[j] = temp


def randbelow(a:int) -> int:
    return random_below(a)

def randint(a:int, b:int) -> int:
    return cy_uniform_int_variate(a, b)



@cython.boundscheck(False)
@cython.wraparound(False)
cpdef object random_choice(object container):
    cdef Py_ssize_t index = cy_random_below(len(container))
    return container[index]

@cython.boundscheck(False)
@cython.wraparound(False)
cpdef list[object] random_choices(object container, Py_ssize_t count):
    cdef Py_ssize_t container_length = len(container)
    return [container[cy_random_below(container_length)] for _ in range(count)]

@cython.boundscheck(False)
@cython.wraparound(False)
cpdef list[object] random_sample(object container, Py_ssize_t count):
    cdef Py_ssize_t container_length = len(container)
    cdef Py_ssize_t i, j
    cdef list result = [None] * count
    cdef list temp = list(container)

    if count > container_length:
        raise ValueError("Sample larger than population")

    for i in range(count):
        j = uniform_int_variate(i, container_length - 1)
        result[i] = temp[j]
        temp[j] = temp[i]

    return result

cpdef int randrange(int start, int stop=0, int step=1):
    if stop == 0:
        stop, start = start, 0
    return random_range(start, stop, step)


cpdef double random_double(double a, double b):
    return uniform_real_variate(a, b)


cpdef double random_double_noargs():
    return uniform_real_variate_noargs()



from typing import Any, Union

def seed(rseed: int) -> None: ...
def shuffle(array: list) -> None:
    """清洗列表中的元素，使其随机排列。

    Args:
        array (list): 要清洗的列表。
    """
    ...

def random_sample(container: Union[list, tuple, str, bytes], count: int) -> list[Any]:
    """从容器中随机抽取count个元素。

    Args:
        container (Union[list, tuple, str, bytes]): 要抽取的容器。
        count (int): 要抽取的元素个数。

    Returns:
        list[Any]: 抽取的元素。

    """

def random_integer_noargs() -> int:
    """生成一个随机整数

    Returns:
        int: 随机整数。
    """
    ...

def randbelow(n: int) -> int:
    """返回一个随机整数，范围是[0, n)。

    Args:
        n (int): 范围的上限。

    Returns:
        int: 随机整数。
    """
    ...

def random_choice(container: Union[list, tuple, str, bytes]) -> Any:
    """从列表、元组、字符串或字节串中随机选择一个元素。

    Args:
        elements (Union[list, tuple, str, bytes]): 要选择的元素。

    Returns:
        Any: 随机选择的元素。
    """
    ...

def random_choices(container: Union[list, tuple, str, bytes], count: int) -> list[Any]:
    """从列表、元组、字符串或字节串中随机选择count个元素。

    Args:
        elements (Union[list, tuple, str, bytes]): 要选择的元素。
        count (int): 选择的元素个数。

    Returns:
        list[Any]: 随机选择的元素。
    """
    ...

def randint(min: int, max: int) -> int:
    """返回一个随机整数，范围是[min, max]。

    Args:
        min (int): 最小值。
        max (int): 最大值。

    Returns:
        int: 随机整数。
    """
    ...

def randrange(start: int, stop: int = 0, step: int = 1) -> int:
    """返回一个随机整数，范围是[start, stop)。

    Args:
        start (int): 开始值。
        stop (int, optional): 结束值. 默认为0。
        step (int, optional): 步进. 默认为1。

    Returns:
        int: 范围内随机整数。
    """
    ...

def random_double(a: float, b: float) -> float:
    """返回一个随机浮点数，范围是[a,b)。

    Args:
        a (float): a float
        b (float): a float

    Returns:
        float: 范围内随机浮点数
    """
    ...

def random_double_noargs() -> float:
    """1~0之间的随机浮点数

    Returns:
        float: 1~0之间的随机浮点数
    """
    ...



# cython: language_level=3
# distutils: language = c++
from libc.stdint cimport int64_t
#cpdef void seed(int rseed = ?)
cdef int64_t cy_random_below(int64_t number) nogil
cdef int64_t cy_uniform_int_variate(int64_t a, int64_t b) nogil
cpdef void shuffle(list[object] array)
cpdef int random_integer_noargs()
cpdef object random_choice(object container)
cpdef list[object] random_choices(object container, Py_ssize_t count)
cpdef list[object] random_sample(object container, Py_ssize_t count)
# def int randbelow(int a)
# def int randint(int a,int b)
cpdef int randrange(int start,int stop=?,int step=?)
cpdef double random_double(double a, double b)
cpdef double random_double_noargs()


from setuptools import setup, Extension
from Cython.Build import cythonize
from sys import platform


def readme():
    with open("README.md") as f:
        return f.read()


extra_compile_args = []
extra_link_args = []

if platform == "win32":
    extra_compile_args = ["/std:c++17", "/O2"]
elif platform == "linux":
    extra_compile_args = ["-std=c++17", "-O3"]
    extra_link_args = ["-Wl,-O3"]
elif platform == "darwin":  # macOS
    extra_compile_args = ["-std=c++17", "-O3"]
    extra_link_args = ["-Wl,-dead_strip"]

setup(
    name="nazo_rand",
    ext_modules=cythonize(
        Extension(
            name="",
            sources=["nazo_rand/nazo_rand.pyx"],
            language=["c++"],
            extra_compile_args=extra_compile_args,
            extra_link_args=extra_link_args,
        ),
        compiler_directives={
            "language_level": 3,
            "boundscheck": False,
            "wraparound": False,
            "binding": True,
            "cdivision": True,
        },
    ),
    author="bymoye",
    author_email="s3moye@gmail.com",
    version="0.0.9",
    description="A fast random number generator for python",
    long_description=readme(),
    long_description_content_type="text/markdown",
    license="Free for non-commercial use",
    package_data={
        "": [
            "nazo_rand/nazo_rand.pyi",
            "nazo_rand/nazo_rand.pyx",
            "nazo_rand/nazo_rand.hpp",
            "nazo_rand/nazo_rand.pxd",
        ]
    },
    include_package_data=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Cython",
        "Programming Language :: C++",
        "Operating System :: POSIX :: Linux",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=3.8",
    packages=["nazo_rand"],
)
